#include <RcppArmadillo.h>
#include <math.h>


using namespace Rcpp;
using namespace arma;

// [[Rcpp::depends(RcppArmadillo)]]

// log-multivariate-Gamma function

double lm_gamma(double p, double x){
  
  NumericVector lm_gamma(p);
  double c=0.25*(p*(p-1))*log(M_PI);
  
  for(double i = 0; i < p; ++i){
    lm_gamma(i) = x-0.5*i;
  }
  return (sum(lgamma(lm_gamma))+c);
  
}

arma::mat sqrt_mat(arma::mat m){
  
  double n = m.n_rows ;
  vec eigval;
  mat eigvec;
  mat sIt1=zeros(n,n);
  
  eig_sym(eigval, eigvec, m);
  
  for(double i = 0; i < n; ++i) {
    sIt1(i,i)=sqrt(eigval(i));
  }
  
  return eigvec*sIt1*trans(eigvec);
}


// [[Rcpp::export]]
List llik_MC_c(arma::vec theta, arma::mat y, arma::cube rv) {
  
  double n = y.n_rows ;
  double p = y.n_cols ;
  // change w and La
  double v = theta(0);
  double b = theta(1);
  double a = theta(2);
  
  arma::mat La=zeros(p,p);
  arma::mat La1=zeros(p,p);
  
  for(double i = 0; i < p; ++i){
    La(i,i)=theta(3);
    La1(i,i)=1/theta(3);
  }
  
  //set w using covariance targeting
  arma::mat w;
  w=rv.subcube(0,0,0,p-1,p-1,0)/n;
  for(double t = 1; t < n; ++t){
    w+=rv.subcube(0,0,t,p-1,p-1,t)/n;
  }
  
  arma::vec yt(p);
  arma::mat Vt,st,rvt;
  arma::cube V = zeros(p,p,n);
  
  long double llik=0.0,llik_rv=0.0,llik_y=0.0;
  long double w_const=0.5*v*p*(-log(2)+log(v))-lm_gamma(p, 0.5*v);
  
  // initialization
  Vt=w;
  V.subcube(0,0,0,p-1,p-1,0)=Vt;
  
  yt=trans(y.submat(0,0,0,p-1));
  rvt=rv.subcube(0,0,0,p-1,p-1,0);
  
  for(double t = 1; t < n; ++t) {
    // score
    st= 1/(v+1)*(v*rvt+La1*yt*trans(yt)*La1)-Vt;
    
    // update
    Vt=w*(1-b)+b*Vt+a*st;
    
    V.subcube(0,0,t,p-1,p-1,t)=Vt;
    yt=trans(y.submat(t,0,t,p-1));
    rvt=rv.subcube(0,0,t,p-1,p-1,t);
    
    //compute log-likelihood
    llik_y+=-0.5*(log(det(2*M_PI*La*Vt*La))+arma::sum(trans(yt)*inv(La*Vt*La)*yt));
    llik_rv+=-0.5*v*(log(det(Vt))+arma::trace(inv(Vt)*rvt))+0.5*(v-p-1)*log(det(rvt))+w_const;
  }
  
  llik = llik_rv+llik_y;
  
  if(llik!=llik){
    llik=-1e10;
  }
  
  return List::create( 
    _["f"]  = V,
    _["llik"]  = -llik,
    _["llik_y"]  = -llik_y, 
    _["llik_rv"]  = -llik_rv); 
}

// [[Rcpp::export]]
List gen_MC_c(arma::vec theta, arma::mat e, arma::cube ev) {
  
  double n = e.n_rows ;
  double p = e.n_cols ;
  
  double v = theta(0);
  double b = theta(1);
  double a = theta(2);
  double w = theta(4);
  
  arma::mat La=eye(p,p)*theta(3);
  arma::mat La1=eye(p,p)*(1/theta(3));
  
  arma::vec yt,et;
  arma::mat Vt,st,evt,rvt,y=zeros(n,p),Ik=eye(p,p);
  arma::cube rv=zeros(p,p,n),V=zeros(p,p,n);
  
  // initialization (unconditional expectation)
  Vt=w/(1-b)*Ik;
  V.subcube(0,0,0,p-1,p-1,0)=Vt;

  et=trans(e.submat(0,0,0,p-1));
  yt=sqrt_mat(La*Vt*La)*et;
  y.submat(0,0,0,p-1)=trans(yt);
  
  evt=ev.subcube(0,0,0,p-1,p-1,0);
  rvt=sqrt_mat(Vt)*evt*sqrt_mat(Vt);
  rv.subcube(0,0,0,p-1,p-1,0)=rvt;
  
  for(double t = 1; t < n; ++t){
    
    // score
    st= 1/(v+1)*(v*rvt+La1*yt*trans(yt)*La1)-Vt;
    // update
    Vt=w*Ik+b*Vt+a*st;
    
    V.subcube(0,0,t,p-1,p-1,t)=Vt;
    
    et=trans(e.submat(t,0,t,p-1));
    yt=sqrt_mat(La*Vt*La)*et;
    y.submat(t,0,t,p-1)=trans(yt);
    
    evt=ev.subcube(0,0,t,p-1,p-1,t);
    rvt=sqrt_mat(Vt)*evt*sqrt_mat(Vt);
    rv.subcube(0,0,t,p-1,p-1,t)=rvt;
    
  }
  
  return List::create( 
    _["V"]  = V,
    _["y"]  = y,
    _["rv"]  = rv); 
}