## load libraries Rcpp and RcppArmadillo (needed as likelihood is written in c++)
## note that Rtools also need to be installed
library(Rcpp)
library(RcppArmadillo)

## call likelihood function from c++ files
sourceCpp("Wishart.cpp")

## Load data (data is stored as a list in dta format)
load("data/rk.dta")    # realized variance
load("data/yoc.dta")   # daily returns

## Select data from the lists using the index "m"
# The index m can be selected from 1 to 21. From 1 to 10 there are the bivariate series and realized covariances as in the paper (same order). 
# From 11 to 20 there are the 5x5 series (same order as in the paper). For m=21 there is the 15x15 series.
m=1
y=yoc[[m]]           
rv=rk[[m]]           
n=dim(y)[1]
p=dim(y)[2]

# set initial parameters for numerical optimization
lambda=rep(1,p)
v=p+10
b=0.95
a=0.3
theta_ini=c(v,b,a,lambda)

## numerical optimization and standard errors
est=optim(par=theta_ini, fn=function(x) llik_c(x,y,rv)$llik, method="BFGS")
se=sqrt(diag(solve(optimHess(est$par, fn=function(x) llik_c(x,y,rv)$llik))))

## print estimation results
print("parameter estimates are:")
round(est$par,3)
print("standard errors are estimates are:")
round(se,3)

# obtain time-varying covariance matrix
Vt=llik_c(est$par,y,rv)$f

# plot sqrt element [1,1] of the dynamic covariance matrix 
plot(sqrt(Vt[1,1,]),type="l")
