## load libraries Rcpp and RcppArmadillo (needed as likelihood is written in c++)
## note that Rtools also need to be installed
library(Rcpp)
library(RcppArmadillo)

## call likelihood function and DGP function from c++ files
sourceCpp("Monte_Carlo.cpp")

# choose the dimension of the dataset
p=5
# choose the time series length
n=500
# set the true parameter values
lambda=1
w=0.1
v=p+10
b=0.97
a=0.30
theta=c(v,b,a,lambda,w)


## generate errors
e_rv=rWishart(n, theta[1], diag(1,p)/theta[1])  #Wishart distribution
e=matrix(rnorm(n*p),n,p)                        #Normal distribution


## genrate series  (the DGP function gen_MC_c requires parameter value and generated errors)
gen_rv=gen_MC_c(theta,e,e_rv)
y=gen_rv$y     # daily returns
rv=gen_rv$rv   # realized variance


## estimation from the generated data
# set initial parameter values for optimization (this case we set to the true one, other values can be used)
theta_ini=c(v,b,a,lambda)

# numerical optimization (the likelihood function is llik_MC_c)
est=optim(par=theta_ini, fn=function(x) llik_MC_c(x,y,rv)$llik, method="BFGS")

# report parameter estimate
print("The parameter estimates are:")
round(est$par,3)
